/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "animation.h"

#include "src/foundation/logger.h"

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <qstring.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qfileinfo.h>

Animation::Animation()
{
	activeFrame = -1;
}


Animation::~Animation()
{
	for(unsigned int i = 0; i < frames.size(); i++) {
		delete frames[i];
	}
}


void Animation::addFrame(char *frameName)
{
	Logger::get().logDebug("Adding frame in Animation:");
	Logger::get().logDebug(frameName);

	Frame *f = new Frame(frameName);
	frames.insert(frames.begin() + activeFrame +1, f);
	this->notifyAdd(f, ++activeFrame);
	this->setActiveFrame(activeFrame);
}


/**
 *@todo refactor the if-sentence by drawing out common code.
 */
void Animation::removeFrame()
{
	if(activeFrame >= 0) {
		if(activeFrame < (int)frames.size()-1) {		//The frame to be deleted is between other frames.
			delete frames[activeFrame];
			frames.erase(frames.begin()+activeFrame);
			this->notifyRemove(activeFrame);
			this->setActiveFrame(activeFrame);
		}
		else {									//The frame to be deleted is the last frame.
			delete frames[activeFrame];
			frames.erase(frames.begin() + activeFrame);
			this->notifyRemove(activeFrame);
			//activeFrame--;
			this->setActiveFrame(--activeFrame);
		}
	}
}


void Animation::moveFrame(unsigned int fromPosition, unsigned int toPosition)
{
	if( (fromPosition < frames.size()) && (toPosition < frames.size()) && (fromPosition != toPosition)) {
		Frame *f = frames[fromPosition];
		frames.erase(frames.begin() + fromPosition);
		frames.insert(frames.begin() + toPosition, f );
		this->notifyMove(fromPosition, toPosition);
		this->setActiveFrame(toPosition);
	}
}


Frame* Animation::getFrame(unsigned int frameNumber)
{
	if(frameNumber < frames.size()) {
		Logger::get().logDebug("Retrieving frame from Animation");
		return frames[frameNumber];
	}
	else {
		Logger::get().logWarning("Requesting a frame which is not " 
				"in the animation (framenumber out of bounds)");
		return NULL;
	}
}


unsigned int Animation::getModelSize() 
{
	return frames.size();
}


void Animation::setActiveFrame(int frameNumber)
{
	this->activeFrame = frameNumber;
	this->notifyNewActiveFrame(frameNumber);
}


void Animation::playFrame(int frameNumber)
{
	notifyPlayFrame(frameNumber);
}


int Animation::getActiveFrameNumber()
{
	return activeFrame;
}


void Animation::clear()
{
	frames.clear();
	activeFrame = -1;
	notifyClear();
}


bool Animation::openProject(const char *filename)
{
	xmlDoc *doc = NULL;
	xmlNode *root_element = NULL;
	bool isLoaded = true;
	
	clear();
		
	LIBXML_TEST_VERSION;
		
	doc = xmlReadFile(filename, NULL, 0);
	if (doc == NULL) {
		Logger::get().logWarning("Couldn't load XML file");
		isLoaded = false;
	}
	
	root_element = xmlDocGetRootElement(doc);
	getAttributes(root_element);
	
	xmlFreeDoc(doc);
	xmlCleanupParser();
	
	return isLoaded;
}


bool Animation::saveProject(const char *directory)
{
	xmlDocPtr doc = NULL;
	xmlNodePtr root_node = NULL, images = NULL, node = NULL;
	xmlDtdPtr dtd = NULL;
	
	LIBXML_TEST_VERSION;
	
	doc = xmlNewDoc(BAD_CAST "1.0");
	root_node = xmlNewNode(NULL, BAD_CAST "smil");
	xmlNewProp(root_node, BAD_CAST "xmlns", BAD_CAST "http://www.w3.org/2001/SMIL20/Language");
	xmlNewProp(root_node, BAD_CAST "xml:lang", BAD_CAST "en");
	xmlNewProp(root_node, BAD_CAST "title", BAD_CAST "Caps");
	xmlDocSetRootElement(doc, root_node);
	
	dtd = xmlCreateIntSubset(doc, BAD_CAST "smil", BAD_CAST "-//W3C//DTD SMIL 2.0//EN",
							BAD_CAST"http://www.w3.org/2001/SMIL20/SMIL20.dtd");
	images = xmlNewChild(root_node, NULL, BAD_CAST "images", NULL);
	
	char *id;
	for (unsigned int i = 0; i < frames.size(); i++) {
		id = frames[i]->getPicture();
		node = xmlNewChild(images, NULL, BAD_CAST "img", NULL);
		xmlNewProp(node, BAD_CAST "src", BAD_CAST id);
	}
		
	bool isSaved = saveDOMToFile(directory, doc);
	xmlFreeDoc(doc);
	xmlCleanupParser();
	
	return isSaved;
}


bool Animation::newProject()
{
	clear();
	// do something
	return true;
}


void Animation::getAttributes(xmlNode *node)
{   
	xmlNode *currNode = NULL;
	for (currNode = node; currNode; currNode = currNode->next) {
		if (currNode->type == XML_ELEMENT_NODE) {
			char *filename = (char*)xmlGetProp(currNode, BAD_CAST "src");
			if (filename) {
				addFrame(filename);
			}
			filename = NULL;
		}
		getAttributes(currNode->children);
	}
}


bool Animation::saveDOMToFile(const char *directory, xmlDocPtr doc)
{
	// FIXME: this should be changed to same name as directory
	char *filename = "project.sto";
	char path[256];
	strcpy(path, directory);
	strcat(path, filename);
	
	int ret = xmlSaveFormatFile( path, doc, 1 );
	if (ret == -1) {
		Logger::get().logWarning("Couldnt save DOM to file");
		return false;
	}
	
	return true;
}


