/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ANIMATIONMODEL_H
#define ANIMATIONMODEL_H

#include "src/config.h"
#include "src/presentation/observer.h"
#include "frame.h"

//STL
#include <vector>



using namespace std;

/**
 *The animation model abstract class. All implementations of the animation 
 *model have to inherit from this class.
 *
 *AnimationModel serves as the Subject in our implementation of the observer
 *pattern, and takes care of notifying the observers when a change occur.
 *
 *@author Bjoern Erik Nilsen & Fredrik Berg Kjoelstad
 */
class AnimationModel
{
public:
	
	virtual ~AnimationModel();

	/**
	 *Attatches a new observer to the model. The observer will be notified when
	 *something is changed in the model.
	 *
	 *@param o the observer to be attatched to the model.
	 */
	void attatch(Observer *o);
	
	/**
	 *Detatches an observer from the model. The observer will no longer be notified 
	 *when something is changed in the model.
	 *
	 *@param o the observer to be detatched from the model.
	 */
	void detatch(Observer *o);
	

	/**
	 * Notify the observers that a frame is added to the model.
	 * @param frame the frame which is added to the model.
	 * @param index the position it is added to.
	 */
	void notifyAdd(Frame* frame, unsigned int index);
	
	
	/**
	 * Notify the observers that a frame is removed from the model.
	 * @param frameNumber the number of the removed frame.
	 */
	void notifyRemove(int frameNumber);
	
	
	/**
	 * Notify the observers that a frame in the model has changed location (moved).
	 * @param fromPosition the position the frame is moved from.
	 * @param toPosition the position the frame is moved to.
	 */
	void notifyMove(unsigned int fromPosition, unsigned int toPosition);
	
	
	/**
	 * Notify the observers the active frame in the model has changed.
	 * @param frameNumber the number of the new active frame.
	 */
	void notifyNewActiveFrame(int frameNumber);
	
	
	/**
	 * Notify the observers that the model has been cleared.
	 */
	void notifyClear();
	
	
	/**
	 * Notify the observers that a frame is to be played.
	 * @param frameNumber the frame to be played.
	 */
	void notifyPlayFrame(int frameNumber);
	
	
	/**
	 *Abstract function which adds a new frame to the AnimationModel.
	 *
	 *@param frameName the name on the storage medium of the frame to
	 *	be added.
	 */
	virtual void addFrame(char *frameName) = 0;
	 
	 
	/**
	 *Abstract funtion which removes the active frame from the animationmodel.
	 */
	virtual void removeFrame() = 0;

	
	/**
	 *Abstract function for moving the frame at fromPosition from to toPosition.
	 *@param fromPosition the position the frame to be moved have.
	 *@param toPosition the position to move the frame to.
	 */
	virtual void moveFrame(unsigned int fromPosition, unsigned int toPosition) = 0;
	
	 
	/**
	 *Abstract function which retrieves a given frame
	 *
	 *@param frameNumber the number of the frame to retrieve
	 *@return the frame to retrieve. 
	 */
	virtual Frame* getFrame(unsigned int frameNumber) = 0;
	 
	 
	/**
	 *Abstract function which returns the size of the model.
	 *@return the size of the model.
	 */
	virtual unsigned int getModelSize() = 0;
	 
	 
	/**
	 *Abstract function for changing the currently active frame.
	 *@param frameNumber the number of the new active frame.
	 */
	 virtual void setActiveFrame(int frameNumber) = 0;
	  
	 
	 /**
	  * Abstract function for playing the frame with number frameNumber.
	  * @param frameNumber the number of the frame to play.
	  */
	 virtual void playFrame(int frameNumber) = 0;
	 
	 
	 /**
	  * Abstract function which returns the number of the currently active frame
	  * in the model.
	  * @return the number of the currently active frame.
	  */
	 virtual int getActiveFrameNumber() = 0;
	  
	 
	/**
	 * Abstract function for clearing the model.
	 */
	virtual void clear() = 0; 


	/**
	 * Abstract function for opening a project.
	 */
	virtual bool openProject(const char *filename) = 0;
	
	/**
	 * Abstract function which saves the active project to a XML-file 
	 * which is written to disk.
	 */
	virtual bool saveProject(const char *directory) = 0;
	
	/**
	 * Abstract funtion to create a new project.
	 */
	virtual bool newProject() = 0;
	 
private:
	/**
	 *The datastructure containing the observers to be notified when something is 
	 *changed in the model.
	 */
	vector<Observer*> observers;
};

#endif
