/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DOMAINFACADE_H
#define DOMAINFACADE_H

#include "src/config.h"
#include "animation/animationmodel.h"

/**
 *Singleton facade for the domain level. All requests into the domain layer
 *has to pass through this facade. The only exception is observer requests
 *for display data.
 *
 *DomainFacade is implemented with lazy allocation.
 *
 *@author Bjoern Erik Nilsen & Fredrik Berg Kjoelstad
 */
class DomainFacade
{
public:
	/**
	 *Static function to retrieve the factory instance object from the singleton 
	 *class.
	 *
	 *@return the factory for sending requests into the domain.
	 */
	static DomainFacade* getFacade();
	
	
	
	/**
	 *Function for retrieving the AnimationModel. This function should only be used
	 *for initializing observers.
	 *
	 *@return the AnimationModel for the program.
	 */
	AnimationModel* getModel();
	
	
	/**
	 *Inserts a new frame into the animationmodel.
	 *
	 *@param frameName the name of the frame to be inserted into the animationmodel
	 */
	void addFrame(char *frameName);
	 
	
	/**
	 *Function to change the currently active frame. (Working frame).
	 *@param frameNumber the number of the new active frame.
	 */
	void setActiveFrame(int frameNumber);   
	

	/**
	 * Returns the number of the currently active frame in the model.
	 * @return 
	 */
	int getActiveFrameNumber();
	
	
	/**
	 * Plays the frame with the number frameNumber
	 * @param frameNumber the number of the frame to play
	 */
	void playFrame(int frameNumber);
	
	
	/**
	 *Removes the active frame from the animation.
	 */
	void removeFrame();
	
	/**
	 * Opens a project.
	 */
	bool openProject(const char *filename);
	
	/**
	 * Saves the active project to a XML-file which is written to disk.
	 */
	bool saveProject(const char *directory);
	
	/**
	 * Creates a new project.
	 */
	bool newProject();
	
	/**
	 *Moves the frame at fromPosition from to toPosition.
	 *@param fromPosition the position the frame to be moved have.
	 *@param toPosition the position to move the frame to.
	 */
	void moveFrame(unsigned int fromPosition, unsigned int toPosition);
	
	/**
	 *Retrieves a given frame.
	 *
	 *@param frameNumber the number of the frame to retrieve.
	 *@return the name for the picturefile of this frame. 
	 */
	 Frame* getFrame(unsigned int frameNumber);
	 
	 
	 /**
	  * Retrieves the size of the model
	  * @return the size of the model
	  */
	 int getModelSize();

protected:
	/**
	 *The constructor. It is protected so that it will be impossible for other classes
	 *who doesn't inherit from it to instanciate the singleton. 
	 */
	DomainFacade();
	
private:
	/**
	 *The singleton instance of this class.
	 */
	static DomainFacade *domainFacade;

	AnimationModel *animationModel;
};

#endif
